/* static char sccsid[] = "@(#)90    1.29  atldd/inc/mtlibio.h, atldd, atldd, 6.8.3.0 2/6/08 14:59:50"; */
/*****************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993, 1996           */
/*  All Rights Reserved                                                      */
/*  Licensed Materials - Property of IBM                                     */
/*                                                                           */
/*  US Government Users Restricted Rights - Use, duplication or              */
/*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*  This header file defines the interface for access to IBM 3494 Automated  */
/*  Tape Libraries using EtherNet, Token Ring, or TTY(RS232) attachments.    */
/*  Applications involving these libraries require this header file when     */
/*  using the standard Unix "open, close, ioctl" interface for performing    */
/*  I/O operations.                                                          */
/*                                                                           */
/*****************************************************************************/
#ifndef MTLIBIO_H
#define MTLIBIO_H

/*****************************************************************************/
/*                                                                           */
/*  VARIOUS CONSTANTS FOR TAPE LIBRARY IOCTL'S                               */
/*                                                                           */
/*****************************************************************************/
#ifdef sun
#include <sys/ioccom.h>
#endif

#ifdef _SEQUENT_
#include <sys/sockio.h>
#endif

/*****************************************************************************/
/*  Function prototypes for the libibm interface                             */
/*****************************************************************************/
int open_ibmatl(char *);
int close_ibmatl(int);
int ioctl_ibmatl(int, unsigned int, void *);

/*****************************************************************************/
/*  Maximum length for sense information                                     */
/*****************************************************************************/
#define  MT_SENSE_LENGTH  32

/*****************************************************************************/
/*  Definitions for versn fields.                                            */
/*****************************************************************************/
#define LIB1_VERSN      0

/*****************************************************************************/
/*  Cancel types for Cancel ioctl                                            */
/*****************************************************************************/
#define CDLA            0x00           /* Cancel Drive Library Activity      */
#define CAHA            0x01           /* Cancal All Host Related Activity   */
#define MIDC            0x02           /* Message ID Cancel                  */

/*****************************************************************************/
/*  Audit types for Library Audit ioctl                                      */
/*****************************************************************************/
#define VOL_AUDIT       0x01           /* Volume Audit                       */

/*****************************************************************************/
/*  Define to specify bypass of assignment checking done by CU for mounts,   */
/*  demounts and set device catory operations                                */
/*****************************************************************************/
#define BYPASS_ASSIGN_CHECK 0x00000001

/*****************************************************************************/
/*  Defines for the "sub_cmd" field of the structure "mtlqarg" and           */
/*  the "info_type" field in the "lib_query_info" strucuture.                */
/*****************************************************************************/
#define MT_QVD         0x01            /* Query Volume Data                  */
#define MT_QLD         0x02            /* Query Library Data                 */
#define MT_QSD         0x03            /* Query Statistical Data             */
#define MT_QID         0x04            /* Query inventory data               */
#define MT_QCID        0x05            /* Query category inventory data      */
#define MT_QDD         0x06            /* Query device data                  */
#define MT_QEVD        0x07            /* Query Expanded volume data         */
#define MT_QIVCD       0x08            /* Query Inventory Volume Count Data  */
#define MT_QRCL        0x09            /* Query Reserved Category List       */
#define MT_QCAL        0x0A            /* Query Category Attribute List      */

/*****************************************************************************/
/*  Defines for the "flags" field of the structure "mtlqarg" for query       */
/*  expanded volume data with usage date included.                           */
/*****************************************************************************/
#define ISO_DATE         0x01            /* Include usage date as yyyy.mm.dd */
#define EUROPEAN_DATE    0x02            /* Include usage date as dd.mm.yyyy */
#define USA_DATE         0x04            /* Include usage date as mm.dd.yyyy */
#define DASH_SEPARATOR   0x08            /* Use dash separator character     */
#define PERIOD_SEPARATOR 0x10            /* Use period separator character   */
#define SLASH_SEPARATOR  0x20            /* Use slash separator character    */

/*****************************************************************************/
/*  Defines for "dev_state" field in the structure "lib_query_info".         */
/*****************************************************************************/
#define MT_INSTALLED         0x80      /* Device installed in ATL.           */
#define MT_AVAILBLE          0x40      /* Dev is available to ATL.           */
#define MT_VOL_LOADED        0x20      /* Volume is loaded                   */
#define MT_ACL_INSTALLED     0x10      /* ACL is installed                   */
#define MT_ACL_ACTIVE        0x08      /* ACL is active -- at minimum,       */
                                       /* there is one cartridge in the ACL  */
#define MT_CAT_ORDER         0x04      /* Pre-loading of ACL is done         */
                                       /* via the order in which volumes     */
                                       /* were assigned to catagory          */
#define MT_AUTO_FILL         0x02      /* Pre_loading is enabled             */
#define MT_NULL_CATEGORY     0x01      /* Device explicitely assigned to     */
                                       /* NULL category by host              */

/*****************************************************************************/
/*  Defines for "ext_device_status" field in the structure "lib_query_info". */
/*****************************************************************************/
#define MT_GENERATE_FIRST_MOUNT  0x80  /* Generate first mount               */
#define MT_ENABLE_AUTO_MOUNT     0x40  /* Enable Auto Mount                  */
#define MT_SUPPRESS_UNSOLICITED  0x20  /* Suppress Unsolicited Interrupts    */

/*****************************************************************************/
/*  Defines for the "dev_class" field in the structure "lib_query_info".     */
/*****************************************************************************/
#define MT_3490_B02_B04       0x00     /* 3490 Model B02/B04                 */
#define MT_3490_B20_B40       0x01     /* 3490 Model B20/B40                 */
#define MT_3490E              0x01     /* 3490E Models C1A, C2A, F1A         */
#define MT_3590_B1A           0x11     /* 3590 Model B1A                     */
#define MT_3590_E1A           0x13     /* 3590 Model E1A                     */
#define MT_3590_H1A           0x14     /* 3590 Model H1A                     */
#define MT_3592_J1A           0x20     /* 3592 Model J1A                     */
#define MT_3592_E05           0x22     /* 3592 Model E05                     */
#define MT_3592_E06           0x24     /* 3592 Model E06                     */

/*****************************************************************************/
/*  Defines for the "vol_state" field in the structure "lib_query_info".     */
/*****************************************************************************/
#define MT_VSLI     0x80      /* Volume present in Library, but Inaccessible */
#define MT_VSMQ     0x40      /* Volume Mounted or Queued for mounting       */
#define MT_VSEP     0x20      /* Volume Eject Pending                        */
#define MT_VSPE     0x10      /* Volume in Process of being Ejected          */
#define MT_VSM      0x08      /* Volume is Misplaced                         */
#define MT_VSUU     0x04      /* Volume has Unreadable label or is Unlabeled */
#define MT_VSMM     0x02      /* Volume was used during Manual Mode          */
#define MT_VSME     0x01      /* Volume was Manually Ejected                 */

/*****************************************************************************/
/*  Defines for the extended "volume_status" field in the structure          */
/*  lib_query_info.                                                          */
/*****************************************************************************/
#define MT_VLI     0x8000     /* Volume present in Library, But Inaccessible */
#define MT_VM      0x4000     /* Volume is mounted                           */
#define MT_VQM     0x2000     /* Volume queued for mounting                  */
#define MT_VPM     0x1000     /* Volume is in the process of being mounted   */
#define MT_VQD     0x0800     /* Volume is queued for demount                */
#define MT_VPD     0x0400     /* Volume is in the process of being demounted */
#define MT_VQE     0x0200     /* Volume is queued for eject                  */
#define MT_VPE     0x0100     /* Volume is in the process of being ejected   */
#define MT_VQA     0x0080     /* Volume is queued for audit                  */
#define MT_VPA     0x0040     /* Volume in the process of auditing           */
#define MT_VMIS    0x0020     /* Volume is Misplaced                         */
#define MT_VUU     0x0010     /* Volume is unreadable or unlabeled           */
#define MT_VMM     0x0008     /* Volume used during manual mode              */
#define MT_VME     0x0004     /* Volume was manually ejected                 */

/*****************************************************************************/
/*  Definitions for the library operational states.                          */
/*****************************************************************************/
#define MT_LIB_AOS      0x8000      /* Automated Operational State           */
#define MT_LIB_POS      0x4000      /* Paused Operational State              */
#define MT_LIB_MMOS     0x2000      /* Manual Mode Operational State         */
#define MT_LIB_DO       0x1000      /* Degraded Operation                    */
#define MT_LIB_SEIO     0x0800      /* Safety Enclose Interlock Open         */
#define MT_LIB_VSNO     0x0400      /* Vision system Non-Operational         */
#define MT_LIB_OL       0x0200      /* Offline                               */
#define MT_LIB_IR       0x0100      /* Intervention Required                 */
#define MT_LIB_CK1      0x0080      /* Library Manager Check 1 Condition     */
#define MT_LIB_SCF      0x0040      /* All Storage Cells Full                */
#define MT_LIB_OCV      0x0020      /* Out of Cleaner Volumes                */
#define MT_LIB_DWD      0x0010      /* Dual Write Disabled                   */
#define MT_LIB_EA       0x0008      /* Environmental Alert -- smoke detected */
#define MT_LIB_MMMOS    0x0004      /* Managed Manual Mode Operational State */
#define MT_LIB_SWITCH   0x0002      /* Library Manager switchover in progress*/
#define MT_LIB_SCRATCH  0x0001      /* Out of physical scratch volumes       */

/*****************************************************************************/
/*  Defines for Input/Output Station status                                  */
/*****************************************************************************/
#define MT_IOS_ACISE  0x80          /* All Convenience Input Stations Empty  */
#define MT_IOS_ISDO   0x40          /* Input Station Door Open               */
#define MT_IOS_ACOSE  0x20          /* All Convenience Output Stations Empty */
#define MT_IOS_ACOSF  0x10          /* All Convenience Output Stations Full  */
#define MT_IOS_IM     0x08          /* I/O Stations in input mode            */
#define MT_IOS_BIOI   0x04          /* Bulk I/O Installed                    */
#define MT_IOS_BOSF   0x02          /* Bulk Output Station Full              */
#define MT_IOS_OSDO   0x01          /* Output Station Door Open              */

/*****************************************************************************/
/*  Status type definitions for Library Query Message ID.                    */
/*****************************************************************************/
#define EXE_TYPE       0x00        /* Message ID is either unknown to system */
                                   /* or is still being executed             */
#define DRM_TYPE       0x01        /* Message ID repesents a command with a  */
                                   /* DRM still available                    */

/*****************************************************************************/
/*  Operation Completion Status for Library Query Message ID.                */
/*****************************************************************************/
#define MT_COMP_UNKNOWN_STS  0x00  /* Request ID has already completed       */
                                   /* or is unknown to the subsystem         */
#define MT_INTERM_STS        0x02  /* Subsystem is unable to determine       */
                                   /* the status of the operation            */
#define MT_PENDING_STS       0x03  /* The operation has not completed        */
                                   /* execution and is still pending         */

/*****************************************************************************/
/*  Defines for the "fill_parm" field in the "mtlsdcarg" structure.          */
/*****************************************************************************/
#define MT_EAF    0x80    /* Enable Auto-Fill. Keep the ACL index stack      */
                          /* filled with volumes from the specified category */
#define MT_ECO    0x40    /* Category Order.  Fill the ACL index stack       */
                          /* by selecting volumes from the specified         */
                          /* category based on the order they were assigned  */
                          /* to the category.                                */
#define MT_CACL   0x20    /* Clear Automatic Cartridge Loader.  Reset the    */
                          /* category assignment for the ACL. If any volumes */
                          /* were pre-loaded, they will be removed and       */
                          /* returned to storage.                            */
#define MT_GFM    0x10    /* Generate First Mount.  An LSDC command with     */
                          /* this parameter active, and a category specified */
                          /* in the "category" field, a mount from that      */
                          /* category is placed onto the mount queue.        */
                          /* No DRM is sent for this mount.  this flag may   */
                          /* be used in conjunction with MT_EAM below        */
#define MT_EAM    0x08    /* Enable Auto Mount.  With Implicit Demount,      */
                          /* a mount from category is placed onto the mount  */
                          /* queue when a tape is unloaded from the drive    */
                          /* No DRM is sent to the host for this mount.      */
#define MT_SUAI   0x04    /* With this parameter active, all unsolicited     */
                          /* attention interrupts are suppressed.            */

/*****************************************************************************/
/*  Defines for the "release_attr" field in the "mtlrcarg" structure.        */
/*****************************************************************************/
#define MT_REC   0x00     /* Release Specified Empty Catetory.  Category     */
                          /* specified in the release_cat field of the       */
                          /* structure is released if it is reserved by this */
                          /* host and is empty                               */

/*****************************************************************************/
/*  Defines for the "attr" field in the mtlscaarg structure.                 */
/*****************************************************************************/
#define MT_NAME   0x01    /*  The attr_data field contains a chararacter     */
                          /*  string that is the name to be assigned to the  */
                          /*  category.                                      */

/*****************************************************************************/
/*  Values for "msg_type" field in "mtlewret" structure.                     */
/*****************************************************************************/
#define NO_MSG            1     /* No message                                */
#define UNSOL_ATTN_MSG    2     /* Unsolicited attention message             */
#define DELAY_RESP_MSG    3     /* Delayed response message                  */

/*****************************************************************************/
/*  Values for "timeout" field in "mtlewarg" structure                       */
/*****************************************************************************/
#define LEWTIME        2   /* Wait until a library unsolicited unit check    */
                           /* or attention message occurs, or until a        */
                           /* timeout period runs out, whichever comes first */

#define QUERY_INFO_MAXLN   1024
#define MSG_INFO_MAXLN     4087
#define ATTR_MAXLN         10
#define IDENT_TOKEN_SIZE   8
#define CAT_NAME_SIZE      10
#define MAX_CAT            15

/*****************************************************************************/
/*                                                                           */
/*  Argument and return information structures for tape library ioctl's      */
/*                                                                           */
/*****************************************************************************/
/*  Passed to the MTIOCLM ioctl system call, in the input argument,          */
/*  is the return information structure "mtlmret".  This structure           */
/*  will be filled in by ioctl upon return.                                  */
/*****************************************************************************/
struct mtlmret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  unsigned int req_id;                   /* Message ID of mount operation    */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLM - Magnetic Tape Library Mount.                     */
/*  This structure defines the argument passed to the MTIOCLM                */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlmarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number - Must be zero               */
  int      device;            /* Device number                               */
  int      wait_flg;          /* Indicates requester wishes to wait or not   */
  unsigned short target_cat;  /* Category the VOLSER is to be assigned to    */
  unsigned short source_cat;  /* Category from which volume is to be mounted */
  char     volser[8];         /* VOLSER to be mounted                        */
  struct   mtlmret mtlmret;   /* Return information structure                */
};

/*****************************************************************************/
/*  Passed to the MTIOCLDM ioctl system call, in the input argument,         */
/*  is the return information structure "mtldret".  This structure           */
/*  will be filled in by ioctl upon return.                                  */
/*****************************************************************************/
struct mtldret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  unsigned int req_id;                   /* Message ID of demount operation  */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLDM - Magnetic Tape Library Demount.                  */
/*  This structure defines the argument passed to the MTIOCLDM               */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtldarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number                              */
  int      device;            /* Device number                               */
  int      wait_flg;          /* Indicates requester wishes to wait or not   */
  unsigned short target_cat;  /* Category the VOLSER is to be assigned to    */
  unsigned short pad;         /* Pad to maintain alignment                   */
  char     volser[8];         /* VOLSER to be demounted                      */
  struct   mtldret mtldret;   /* Return information structure                */
};

/*****************************************************************************/
/*  Passed to the MTIOCLC ioctl system call, in the input argument,          */
/*  is the return information structure "mtlcret".  This structure           */
/*  will be filled in by ioctl upon return.                                  */
/*****************************************************************************/
struct mtlcret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      device;                       /* Device cancel performed on       */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLC - Magnetic Tape Library Cancel.                    */
/*  This structure defines the argument passed to the MTIOCLC                */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlcarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number                              */
  int      device;            /* Device number                               */
  unsigned int req_id;        /* Message ID of an async op to be cancelled   */
  int      cancel_type;       /* Type of cancel request                      */
  struct   mtlcret mtlcret;   /* Return information structure                */
};

/*****************************************************************************/
/*  Passed to the MTIOCLSVC ioctl system call, in the input argument,        */
/*  is the return information structure "mtlsvcret".  This structure         */
/*  will be filled in by the ioctl upon return.                              */
/*****************************************************************************/
struct mtlsvcret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  unsigned int req_id;                   /* Message ID of the set volume     */
                                         /* category operation               */
  int      device;                       /* Device the set volume category   */
                                         /* Operation was performed on.      */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLSVC - Magnetic Tape Library Set Volume               */
/*  Category. This structure defines the argument passed to                  */
/*  the MTIOCLSVC ioctl system call.                                         */
/*****************************************************************************/
struct mtlsvcarg {
  int      flags;              /* Flags for special options                  */
  int      versn;              /* Version number                             */
  int      device;             /* Device number                              */
  int      wait_flg;           /* Indicates requester wishes to wait         */
  unsigned short target_cat;   /* Category the VOLSER is to be assigned to   */
  unsigned short source_cat;   /* Category VOLSER is currently assigned to   */
  char     volser[8];          /* VOLSER to be set in a category             */
  struct   mtlsvcret mtlsvcret;/* Return information structure               */
};

/*****************************************************************************/
/*  Passed to the MTIOCLA ioctl system call, in the input argument,          */
/*  is the return information structure "mtlaret".  This structure           */
/*  will be filled in by the ioctl upon return.                              */
/*****************************************************************************/
struct mtlaret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  unsigned int req_id;                   /* Message ID for audit operation   */
  int      device;                       /* Device used for audit operation  */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLA - Magnetic Tape Library Audit.                     */
/*  This structure defines the argument passed to the MTIOCLA                */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlaarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number                              */
  int      device;            /* Device number                               */
  int      wait_flg;          /* Indicates requester wishes to wait or not   */
  int      audit_type;        /* Audit type                                  */
  char     volser[8];         /* VOLSER to be audited                        */
  struct   mtlaret mtlaret;   /* Return information structure                */
};

struct inv_recs {             /* Up to 100 inventory records                 */
  unsigned char cat[2];       /* Category value butes 0-1                    */
  char     volser[6];         /* Volser bytes 2-7                            */
  unsigned char attr[2];      /* Attribute bytes 8-9 - state,class, and type */
};

struct cat_attr_recs {        /* Up to 15 categories attributes in list      */
  unsigned char cat[2];       /* Category value -- bytes 0-1                 */
  unsigned char name[10];     /* Category name  -- bytes 2-11                */
  unsigned char resvd[52];    /* Reserved       -- bytes 12-63               */
};

struct lib_query_info {
  char    info_type;                /* Type of information queried           */
  union   {
    struct    {                     /* Volume specific data                  */
      char     vol_state;           /* Attribute 1 of Volume data            */
                                    /* See defines above.                    */
      char     vol_class;           /* Attribute 2 of Volume data            */
                                    /* bit 0    Logical Volume               */
                                    /* bit 1    Logical Volume Imported      */
                                    /* bit 2:3  Volume Class                 */
                                    /*          B'00' 3480 1/2 inch cart     */
                                    /*          B'01' 3590 1/2 inch cart     */
                                    /*          B'10' 3592 1/2 inch cart     */
                                    /* bit 4:7  Volume Type                  */
                                    /*          B'0000' 160 m nominal or     */
                                    /*                  320 m nominal 3590   */
                                    /*                  3592 JA cart         */
                                    /*          B'0001' 320 m nominal or     */
                                    /*                  extended length 3590 */
                                    /*                  3592 JW cart         */
                                    /*          B'0010' 3592 JJ cart         */
                                    /*          B'0011' 3592 JR cart         */
                                    /*          B'0100' 3592 JB cart         */
                                    /*          B'0101' 3592 JX cart         */
                                    /*          B'1111' Unknown length       */
      char     volser[6];           /* Volume serial number                  */
      unsigned char category[2];    /* Category Assigned                     */
      char     affinity_lst[32];    /* Subsystem Affinity List               */
                                    /* Up to 32 subsystems -- 0x00           */
                                    /* indicates end of list.                */
    } volume_data;

    struct {                        /* Library Data                          */
      unsigned char op_state[2];    /* Operational State - see defines       */
                                    /* for library states.                   */
      char     func_state;          /* Functioanl State                      */
                                    /* bit 0 Host initiated export in progres*/
                                    /* bit 1 reserved                        */
                                    /* bit 2 Host initiated import in progres*/
                                    /* bit 3 Library initiated import in prog*/
                                    /* bit 4:7 reserved                      */
      char     inout_no;            /* Number of input/output stations       */
                                    /* bit0-3 input, bit4-7 out              */
      char     in_out_status;       /* Input/Output Station Status - see     */
                                    /* defines above for I/O station status  */
      unsigned char lib_mach_type[2]; /* Libary machine type                 */
      char     res2;
      unsigned char atl_seqno[3];   /* Library Sequence Number               */
      unsigned char num_cells[4];   /* Number of storage cells in library    */
      unsigned char avail_cells[4]; /* Number of available storage cells     */
      char     num_subsys;          /* Number of subsystems in the library   */
      char     conven_cap;          /* Number of cartridge positions in each */
                                    /* convenience station.                  */
      char     res3;
      char     accessor_config;     /* Accessor configuration                */
                                    /* X'01' - Type 1 accessor configuration */
                                    /* Accessor states -- only defined       */
                                    /* when config type is X'10' uninstalled */
                                    /* accessors are indicated via X'00'.    */
      char     acces0_stat;         /* Accessor zero status                  */
      char     acces1_stat;         /* Accessor one status                   */
      char     acces2_stat;         /* Accessor two status                   */
      char     acces3_stat;         /* Accessor three status                 */
      char     acces4_stat;         /* Accessor four status                  */
      char     acces5_stat;         /* Accessor five status                  */
      char     acces6_stat;         /* Accessor six status                   */
      char     acces7_stat;         /* Accessor seven status                 */
                                    /*                                       */
                                    /* Accessor format                       */
                                    /* ---------------                       */
                                    /* bit 0:1  Accessor State               */
                                    /*     0    Not installed                */
                                    /*     1    Not operational              */
                                    /*     2    Not available                */
                                    /*     3    Available                    */
                                    /*                                       */
                                    /* bit 2:3  Gripper 0 state              */
                                    /*     0    Not installed                */
                                    /*     1    Not operational              */
                                    /*     2    Not available                */
                                    /*     3    Available                    */
                                    /*                                       */
                                    /* bit 4    Gripper 0 vision system      */
                                    /*          operational                  */
                                    /*                                       */
                                    /* bit 5:6  Gripper 1 state              */
                                    /*     0    Not installed                */
                                    /*     1    Not operational              */
                                    /*     2    Not available                */
                                    /*     3    Available                    */
                                    /*                                       */
                                    /* bit 7    Gripper 1 vision system      */
                                    /*          operational                  */
      unsigned char comp_avail_status[2]; /* Component Availability Status   */
                                    /* bit                                   */
                                    /* 0  Primary library mgr installed.     */
                                    /* 1  Primary library mgr available.     */
                                    /* 2  Secondary library mgr installed.   */
                                    /* 3  Secondary library mgr available.   */
                                    /* 4  Primary hard drive installed.      */
                                    /* 5  Primary hard drive available.      */
                                    /* 6  Secondary hard drive installed.    */
                                    /* 7  secondary hard drive available.    */
                                    /* 8  Convenience input station          */
                                    /*    installed.                         */
                                    /* 9  Convenience input station          */
                                    /*    available.                         */
                                    /* 10 Convenience output station         */
                                    /*    installed.                         */
                                    /* 11 Convenience output station         */
                                    /*    available.                         */
                                    /* 12 Removable bulk input station       */
                                    /*    installed.                         */
                                    /* 13 Removable bulk input station       */
                                    /*    available.                         */
                                    /* 14 Removable bulk output station      */
                                    /*    installed.                         */
                                    /* 15 Removable bulk output station      */
                                    /*    available.                         */
      char     lib_facilities;      /* Library Facilities                    */
                                    /* bit                                   */
                                    /* 0  VTS 3590 device unavailable        */
                                    /* 1  Library contains a VTS             */
                                    /* 2  VTS is Export/Import capable       */
                                    /* 3  Export/Import resources available  */
                                    /* 4  Conv I/O station in import mode    */
                                    /* 5  reserved                           */
                                    /* 6  VTS is part of an HA/RCS system    */
                                    /* 7  VTS SCSI host attachemnt installed */
      char     res5[1];
      unsigned char bulk_input_capacity[2];      /* bulk input capacity      */
      unsigned char bulk_input_empty[2];         /* bulk input empty cells   */
                                                 /* X'FFFF' if unknown       */
      unsigned char bulk_output_capacity[2];     /* bulk output capacity     */
      unsigned char bulk_output_empty[2];        /* bulk output empty cells  */
                                                 /* X'FFFF' if unknown       */
      unsigned char avail_3490_cleaner_cycles[2]; /* available cleaner cycles */
      unsigned char avail_3590_cleaner_cycles[2]; /* available cleaner cycles */
      char     distributed_lib_id;   /* Distributed Library ID               */
      char     res6[1];
      unsigned char ext_op_state[2]; /* Extended Operational State           */
      char     pools_out_of_scratch; /* Pools with Out-of-Scratch-Stacked Cond */
      char     pools_volumes;        /* Pools with Volumes                   */
      unsigned char avail_3592_cleaner_cycles[2]; /* available cleaner cycles */
    } library_data;

    struct {                        /* Statistical Data                      */
      char     res1;                /* Reserved                              */
      unsigned char hour_index[2];  /* Hour index                            */
      char machine_type[6];         /* Library machine type                  */
      char model[3];                /* Library model number                  */
      char manufacturer[3];         /* Library manufacturer                  */
      char plant[2];                /* Plant of manufacturer                 */
      char lib_seqno[12];           /* Libarary Sequence Number              */
      /****** Device Statistics ********* (last hour) ************************/
      unsigned char no_drvs[2];          /* no. of installed drives          */
      unsigned char no_mnt_drvs[2];      /* no. of mounted drives            */
      unsigned char max_no_mnts[2];      /* Max no. of drives mounted        */
      unsigned char min_no_mnts[2];      /* Min no. of drives mounted        */
      unsigned char ave_no_mnts[2];      /* Average no. of drvs mounted      */
      unsigned char max_mnted_time[2];   /* Max mounted time                 */
      unsigned char min_mnted_time[2];   /* Min mounted time                 */
      unsigned char ave_mnted_time[2];   /* Average mounted time             */
      /****** Mount Statistics ********** (last hour) ************************/
      unsigned char pending_mnts[2];     /* Pending mount requests           */
      unsigned char max_pend_mnt[2];     /* Max Pending mount requests       */
      unsigned char min_pend_mnt[2];     /* Min pending mount requests       */
      unsigned char ave_pend_mnt[2];     /* Average pending mount requests   */
      unsigned char no_mnts_hr[2];       /* No. of mounts per hour           */
      unsigned char no_idx_mnts_hr[2];   /* No. index mounts per hour        */
      unsigned char no_pre_mnts_hr[2];   /* No. of pre-mounts per hour       */
      unsigned char max_mnt_time[2];     /* Maximum mount time               */
      unsigned char min_mnt_time[2];     /* Minimum mount time               */
      unsigned char ave_mnt_time[2];     /* Average mount time               */
      /****** Demount Statistics ******** (last hour) ************************/
      unsigned char pending_dmnts[2];    /* Pending demount requests         */
      unsigned char max_pend_dmnt[2];    /* Maximum Pending demount requests */
      unsigned char min_pend_dmnt[2];    /* Minimum pending demount requests */
      unsigned char ave_pend_dmnt[2];    /* Average pending demount requests */
      unsigned char no_dmnts_hr[2];      /* No. of demounts per hour         */
      unsigned char no_idx_dmnts_hr[2];  /* No. index demounts per hour      */
      unsigned char no_post_dmnts_hr[2]; /* No. of post-mounts per hour      */
      unsigned char max_dmnt_time[2];    /* Maximum demount time             */
      unsigned char min_dmnt_time[2];    /* Minimum demount time             */
      unsigned char ave_dmnt_time[2];    /* Average demount time             */
      /****** Eject Statistics ********** (last hour) ************************/
      unsigned char pending_ejects[2];   /* Pending eject requests           */
      unsigned char max_pend_ejects[2];  /* Maximum pending eject requests   */
      unsigned char min_pend_ejects[2];  /* Minimum pending eject requests   */
      unsigned char ave_pend_ejects[2];  /* Average pending eject reuests    */
      unsigned char no_ejects_hr[2];     /* No. of ejects per hour           */
      unsigned char max_eject_time[2];   /* Maximum eject time               */
      unsigned char min_eject_time[2];   /* Minimum eject time               */
      unsigned char ave_eject_time[2];   /* Average eject time               */
      /****** Audit Statistics ********** (last hour) ************************/
      unsigned char pending_audits[2];   /* Pending audit requests           */
      unsigned char max_pend_audit[2];   /* Maximum pending audit requests   */
      unsigned char min_pend_audit[2];   /* Minimum pending audit requests   */
      unsigned char ave_pend_audit[2];   /* Average pending audit requests   */
      unsigned char no_audit_hr[2];      /* Number of audits per hour        */
      unsigned char max_audit_time[2];   /* Maximum audit time               */
      unsigned char min_audit_time[2];   /* Minimum audit time               */
      unsigned char ave_audit_time[2];   /* Average audit time               */
      /****** Input Statistics ********** (last hour) ************************/
      unsigned char no_input_str_hr[2];  /* No. of input stores per hour     */
      char     res3[128];                /* Reserved                         */
    } stats_data;

    struct {                        /* Inventory data                        */
      char     lib_seqno[3];        /* Libarary Sequence Number              */
      unsigned char no_vols[4];     /* Number of VOLSERS in library          */
      unsigned char res1[2];        /* Reserved                              */
      struct inv_recs inv_rec[100]; /* 100 inventory records                 */
                                    /* byte 0-1 Category value               */
                                    /* byte 2-7 VOLSER                       */
                                    /* byte 8   Attribute 1                  */
                                    /*           bit 0 - Present but         */
                                    /*                   inaccessible.       */
                                    /*           bit 1 - Mounted or          */
                                    /*                   queued for          */
                                    /*                   mount.              */
                                    /*           bit 2 - Eject pending.      */
                                    /*           bit 3 - In process          */
                                    /*                   of eject.           */
                                    /*           bit 4 - Misplaced.          */
                                    /*           bit 5 - Unreadable label    */
                                    /*                   or unlabeled.       */
                                    /*           bit 6 - Used during         */
                                    /*                   manual mode.        */
                                    /*           bit 7 - Manually Ejected.   */
                                    /* byte 9   Attribute 2                  */
                                    /* bit 0    Logical Volume               */
                                    /* bit 1    Logical Volume Imported      */
                                    /* bit 2:3  Volume Class                 */
                                    /*          B'00' 3480 1/2 inch cart     */
                                    /*          B'01' 3590 1/2 inch cart     */
                                    /*          B'10' 3592 1/2 inch cart     */
                                    /* bit 4:7  Volume Type                  */
                                    /*          B'0000' 160 m nominal or     */
                                    /*                  320 m nominal 3590   */
                                    /*                  3592 JA cart         */
                                    /*          B'0001' 320 m nominal or     */
                                    /*                  extended length 3590 */
                                    /*                  3592 JW cart         */
                                    /*          B'0010' 3592 JJ cart         */
                                    /*          B'0011' 3592 JR cart         */
                                    /*          B'0100' 3592 JB cart         */
                                    /*          B'0101' 3592 JX cart         */
                                    /*          B'1111' Unknown length       */
       unsigned char res2[2];       /* Reserved                              */
    } inventory_data;

    struct {                        /* Category Inventory                    */
      char     lib_seqno[3];        /* Libarary Sequence Number              */
      unsigned char no_vols[4];     /* Number of VOLSERS in Category         */
      unsigned char category[2];    /* Category                              */
      struct inv_recs inv_rec[100]; /* 100 inventory records                 */
                                    /* byte 0-1 Category/Construct           */
                                    /*          Request X'00'  category      */
                                    /*          Request X'01'  constructs    */
                                    /*           bit 0 - No set to the       */
                                    /*                   default name        */
                                    /*           bit 1 - Storage Group       */
                                    /*                   Overridden (Import) */
                                    /*           bit 2 - Management Class    */
                                    /*                   Overridden (Import) */
                                    /*           bit 3 - Storage Class       */
                                    /*                   Overridden (Import) */
                                    /*           bit 4 - Data Class          */
                                    /*                   Overridden (Import) */
                                    /*           bit 5:15 Reserved           */
                                    /* byte 2-7 VOLSER                       */
                                    /* byte 8   Attribute 1                  */
                                    /*           bit 0 - Present but         */
                                    /*                   inaccessible.       */
                                    /*           bit 1 - Mounted or          */
                                    /*                   queued for          */
                                    /*                   mount.              */
                                    /*           bit 2 - Eject pending.      */
                                    /*           bit 3 - In process          */
                                    /*                   of eject.           */
                                    /*           bit 4 - Misplaced.          */
                                    /*           bit 5 - Unreadable label    */
                                    /*                   or unlabeled.       */
                                    /*           bit 6 - Used during         */
                                    /*                   manual mode.        */
                                    /*           bit 7 - Manually Ejected.   */
                                    /* byte 9   Attribute 2                  */
                                    /* bit 0    Logical Volume               */
                                    /* bit 1    Logical Volume Imported      */
                                    /* bit 2:3  Volume Class                 */
                                    /*          B'00' 3480 1/2 inch cart     */
                                    /*          B'01' 3590 1/2 inch cart     */
                                    /*          B'10' 3592 1/2 inch cart     */
                                    /* bit 4:7  Volume Type                  */
                                    /*          B'0000' 160 m nominal or     */
                                    /*                  320 m nominal 3590   */
                                    /*                  3592 JA cart         */
                                    /*          B'0001' 320 m nominal or     */
                                    /*                  extended length 3590 */
                                    /*                  3592 JW cart         */
                                    /*          B'0010' 3592 JJ cart         */
                                    /*          B'0011' 3592 JR cart         */
                                    /*          B'0100' 3592 JB cart         */
                                    /*          B'0101' 3592 JX cart         */
                                    /*          B'1111' Unknown length       */
      unsigned char cat_seqno[4];   /* Catagory sequence number              */
    } cat_invent_data;

    struct {                        /* Device Data                           */
      char     res1;                /* Reserved                              */
      char     volser[6];           /* Mounted VOLSER                        */
      unsigned char mnted_cat[2];   /* Mounted Category                      */
      unsigned char dev_cat[2];     /* Assigned device category              */
      char     dev_state;           /* Device states                         */
                                    /* Possible Values are:                  */
                                    /* MT_INSTALLED - Device installed       */
                                    /* MT_AVAILBLE - Device is available     */
                                    /* MT_VOL_LOADED - Volume is loaded      */
                                    /* MT_ACL_INSTALLED - ACL is installed   */
                                    /* MT_ACL_ACTIVE - ACL is active, there  */
                                    /*                 is at least one       */
                                    /*                 cartridge in the ACL. */
                                    /* MT_CAT_ORDER - pre-loading of ACL is  */
                                    /*                done via the order the */
                                    /*                volumes were assigned  */
                                    /*                to the category        */
                                    /* MT_AUTO_FILL - pre_loading is enabled */
                                    /* MT_NULL_CATEGORY - NULL category      */
                                    /*                    assigned by host   */
      char     dev_class;           /* Device Class, possible values are:    */
                                    /* MT_3490_B02_B04                       */
                                    /* MT_3490_B20_B40                       */
                                    /* MT_3490_C1A_C2A_F1A                   */
                                    /* MT_3590_B1A                           */
                                    /* MT_3590_E1A                           */
                                    /* MT_3590_H1A                           */
                                    /* MT_3592_J1A                           */
                                    /* MT_3592_E05                           */
      char     ext_device_status;   /* Extended Device Status, values are:   */
                                    /* MT_GENERATE_FIRST_MOUNT               */
                                    /* MT_ENABLE_AUTO_MOUNT                  */
                                    /* MT_SUPPRESS_UNSOLICITED               */
      char     res2[9];             /* Reserved                              */
    } dev_data;

    struct {                        /* Expanded Volume Data                  */
      char     res1;                /* Reserved                              */
      unsigned char volume_status[2]; /* Status of the volume,               */
                                    /* possible values:                      */
                                    /* MT_VLI,  MT_VM,   MT_VQM,  MT_VPM,    */
                                    /* MT_VPD,  MT_VQE,  MT_VPE,  MT_VQA,    */
                                    /* MT_VMIS, MT_VUU,  MT_VMM,  MT_VME,    */
                                    /* MT_VQD,  MT_VPA                       */
      char     vol_attr1;           /* Volume attribute                      */
                                    /* bit 0    Logical Volume               */
                                    /* bit 1    Logical Volume Imported      */
                                    /* bit 2:3  Volume Class                 */
                                    /*          B'00' 3480 1/2 inch cart     */
                                    /*          B'01' 3590 1/2 inch cart     */
                                    /*          B'10' 3592 1/2 inch cart     */
                                    /* bit 4:7  Volume Type                  */
                                    /*          B'0000' 160 m nominal or     */
                                    /*                  320 m nominal 3590   */
                                    /*                  3592 JA cart         */
                                    /*          B'0001' 320 m nominal or     */
                                    /*                  extended length 3590 */
                                    /*                  3592 JW cart         */
                                    /*          B'0010' 3592 JJ cart         */
                                    /*          B'0011' 3592 JR cart         */
                                    /*          B'0100' 3592 JB cart         */
                                    /*          B'0101' 3592 JX cart         */
                                    /*          B'1111' Unknown length       */
       char     res2;               /* Reserved                              */
       char     volser[6];          /* Volume serial number                  */
       unsigned char cat_assigned[2]; /* Category assigned to                */
       char     usage_date[10];     /* Date last used                        */
    } expand_vol_data;

    struct {                        /* Inventory Volume Count                */
      unsigned char atl_seqno[3];   /* Library sequence number               */
      unsigned char num_volumes[4]; /* Number of volumes                     */
      unsigned char category[2];    /* Category specified in operation       */
    } invent_vol_cnt;

    struct {                        /* Reserve Category List                 */
      unsigned char atl_seqno[3];   /* Library sequence number               */
      char     ident_token[8];      /* Token for reserved categories         */
      unsigned char count[2];       /* Total number of categories in list    */
      unsigned char cat[256][2];    /* Reserved category record              */
    } reserve_cat_list;

    struct {
      unsigned char atl_seqno[3];   /* Library sequence number               */
      char     ident_token[8];      /* Token for reserved categories         */
      unsigned char count[2];       /* Total number of categories in list    */
      struct   cat_attr_recs cat_attr_rec[15]; /* Category attribute record  */
    } cat_attr_list;

    char     reserved[QUERY_INFO_MAXLN];

  } data;
};

/*****************************************************************************/
/*  Passed to the MTIOCLQ ioctl system call, in the input argument,          */
/*  is the return information structure "mtlqret".  This structure           */
/*  will be filled in by the ioctl upon return.                              */
/*****************************************************************************/
struct mtlqret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      device;                       /* device query was performed on    */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
  struct   lib_query_info info;          /* Query information                */
};

/*****************************************************************************/
/*  Structure for MTIOCLQ - Magnetic Tape Library Query.                     */
/*  This structure defines the argument passed to the MTIOCLQ                */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlqarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number                              */
  int      device;            /* Device number                               */
  int      cat_seqno;         /* Category Sequence Number                    */
  int      sub_cmd;           /* Sub command, possible values are:           */
                              /* MT_QVD   - Query Volume Data                */
                              /* MT_QLD   - Query Library Data               */
                              /* MT_QSD   - Query Statistical Data           */
                              /* MT_QID   - Query inventory data             */
                              /* MT_QCID  - Query category inventory data    */
                              /* MT_QDD   - Query device data                */
                              /* MT_QEVD  - Query Expanded volume data       */
                              /* MT_QIVCD - Query Inventory Volume Count data*/
                              /* MT_QRCL  - Query Reserved Category List     */
                              /* MT_QCAL  - Query Category Attribute List    */
  unsigned short source_cat;  /* Source or starting category                 */
  unsigned short cat_to_read; /* Category to read for MT_QCAL sub command    */
  char     hostid[8];         /* Host id for MT_QRCL and MT_QCAL subcommands */
  char     volser[8];         /* VOLSER                                      */
  struct   mtlqret mtlqret;   /* Return information structure                */
};

/*****************************************************************************/
/*  Passed to the MTIOCLQMID ioctl system call, in the input argument,       */
/*  is the return information structure "mtlqmidret".  This structure        */
/*  will be filled in by ioctl upon return.                                  */
/*****************************************************************************/
struct qmid_info {
  char       status_type;        /* Type of status presented in status field */
                                 /* This may be one of the following values: */
                                 /* EXE_TYPE - the command is either still   */
                                 /*            executing or is unknown to    */
                                 /*            to the subsystem              */
                                 /* DRM_TYPE - referenced message id refers  */
                                 /*            to command with delayed       */
                                 /*            response info still avail.    */
  union {
    char      status;            /* Status for EXE_TYPE                      */
    struct {
      char     drm_type;         /* Can be one of:                           */
                                 /*      MT_NTF_DEL_MC - mount complete      */
                                 /*      MT_NTF_DEL_DC - demount complete    */
                                 /*      MT_NTF_DEL_AC - audit complete      */
                                 /*      MT_NTF_DEL_EC - eject complete      */
      char     cc;               /* Completion code for DRM operation        */
      unsigned char operational_flags; /* Operational flags                  */
      unsigned char category[2]; /* Category                                 */
      char     op_volser[6];     /* Operation VOLSER                         */
      char     read_volser[6];   /* VOLSER read by vision system             */
    } drm_status;
  } status_response;
};

struct mtlqmidret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      device;                       /* Device used for operation        */
  int      status;                       /* Operational Completion status    */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
  struct   qmid_info info;               /* Information returned from query  */
};

/*****************************************************************************/
/*  Structure for MTIOCLQMID - Library Query Message ID.                     */
/*  This structure defines the argument passed to the MTIOCLQMID             */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlqmidarg {
  int      flags;                 /* Flags for special options               */
  int      versn;                 /* Version number                          */
  int      device;                /* Device number                           */
  unsigned int req_id;            /* The message ID associated with an       */
                                  /* asynchronous operation                  */
  struct   mtlqmidret mtlqmidret; /* Return information structure            */
};

/*****************************************************************************/
/*  Passed to the MTIOCLSDC ioctl system call, in the input argument,        */
/*  is the return information structure "mtlsdcret".  This structure         */
/*  will be filled in by ioctl upon return.                                  */
/*****************************************************************************/
struct mtlsdcret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for MTIOCLSDC - Library Set Device Category.                   */
/*  This structure defines the argument passed to the MTIOCLSDC              */
/*  ioctl system call.                                                       */
/*****************************************************************************/
struct mtlsdcarg {
  int      flags;                  /* Flags for special options              */
  int      versn;                  /* Version number                         */
  int      device;                 /* Device number                          */
  int      fill_parm;              /* Fill parameters                        */
  unsigned short category;         /* Category to be assigned to the device  */
  unsigned short demount_category; /* Pad to maintain alignment              */
  struct   mtlsdcret mtlsdcret;    /* Return information structure           */
};

struct cat_info {
  unsigned char atl_seqno[3];     /* Library sequence number                 */
  char     ident_token[8];        /* Token for reserved categories           */
  unsigned char count[2];         /* Total number of categories in list      */
  unsigned char cat[256][2];      /* Reserved category records               */
};

/*****************************************************************************/
/*  Passed to the MTIOCLRC ioctl system call.  This structure is filled      */
/*  in by the ioctl upon return.                                             */
/*****************************************************************************/
struct mtlrcret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
  struct   cat_info info;
};

/*****************************************************************************/
/*  Structure for MTIOCLRC - Library Release Catetory.  This structure       */
/*  defines the arguments passed to the MTIOCLRC ioctl call.                 */
/*****************************************************************************/
struct mtlrcarg {
  int      flags;                  /* Flags for special options              */
  int      versn;                  /* Version number                         */
  int      device;                 /* Device number                          */
  unsigned short release_cat;      /* Category to be released                */
  unsigned short pad;              /* Maintain padding                       */
  char     hostid[8];              /* Host requesting release                */
  struct   mtlrcret mtlrcret;      /* Return information structure           */
};

/*****************************************************************************/
/*  Passed to the MTIOCLRSC ioctl call.  This structure is filled            */
/*  in by the ioctl upon return.                                             */
/*****************************************************************************/
struct mtlrscret {
  int    cc;                           /* Completion code                    */
  int    up_comp;                      /* Reserved                           */
  int    number_sense;                 /* Number of valid sense bytes        */
  char   sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                   */
  struct cat_info info;
};

/*****************************************************************************/
/*  Structure for the MTIOCLRSC - Library Reserve Category ioctl.  This      */
/*  structure defines the arguments passed to the ioctl system call.         */
/*****************************************************************************/
struct mtlrscarg {
  int      flags;                 /* Flags for special options               */
  int      versn;                 /* Version number                          */
  int      device;                /* Device number                           */
  unsigned short num_cat;         /* Number of categories to reserve         */
  unsigned short category;        /* Category to reserve if num_cat is 1     */
  char     hostid[8];             /* Host id requesting reserve              */
  struct   mtlrscret mtlrscret;   /* Return information structure            */
};

/*****************************************************************************/
/*  Passed to the MTIOCLSCA ioctl system call.  This structure is filled     */
/*  in by the the ioctl upon return.                                         */
/*****************************************************************************/
struct mtlscaret {
  int      cc;                           /* Completion code                  */
  int      up_comp;                      /* Reserved                         */
  int      number_sense;                 /* Number of valid sense bytes      */
  char     sense_bytes[MT_SENSE_LENGTH]; /* Sense bytes read                 */
};

/*****************************************************************************/
/*  Structure for the MTIOCLSCA - Library Set Category Attribute ioctl.      */
/*  This structure defines the arguments passed to the ioctl system call.    */
/*****************************************************************************/
struct mtlscaarg {
  int      flags;                 /* Flags for special options               */
  int      versn;                 /* Version number                          */
  int      device;                /* Device number                           */
  unsigned short attr;            /* Attribute description                   */
  unsigned short category;        /* Category whose attribute to set         */
  char     attr_data[ATTR_MAXLN]; /* Data to assign to the category          */
  struct   mtlscaret mtlscaret;   /* Return information structure            */
};

struct msg_info {
  unsigned short  length;    /* Total number of bytes in the message         */
  char     format;           /* Set to X'10' for Automated Tape Library Info */
  char     msg_code;         /* Message code                                 */
  int      msg_id;           /* Message ID                                   */
  char     flags;            /* Flags                                        */
  char     pad[3];           /* Pad to maintain alignment                    */
  union {
    char     resvd[MSG_INFO_MAXLN];

    struct {
      char     resvd;                     /* Reserved                        */
      unsigned char category[2];          /* Category that has been added    */
    } category_state_chg;

    struct {
      char     operator_msg[256];         /* Operator message                */
    } lib_mgr_operator_msg;

    struct {
      char msg_data[256];                 /* Library operation complete      */
    } operation_complete;

    struct {
      char     resvd;              /* Reserved                              */
      char     io_stations;        /* Bits 0-3 input stations               */
                                   /* Bits 4-7 output stations              */
      char     io_station_status;  /* Input/Output Station Status - see     */
                                   /* defines above for I/O station status  */
      char era_code;               /* X'00' - going to NOT_FULL             */
                                   /* X'69' - going FULL                    */
      char modifier;               /* Modifier byte                         */
      unsigned char first_error_code[2];    /* Fist error code              */
      unsigned char last_error_code[2];     /* Last error code              */
    } io_station_state_chg;

    struct {
      unsigned char operational_state[2]; /* Operational state               */
      char     era_code;                  /* Error recovery action code      */
      char     modifier;                  /* Modifier byte                   */
      unsigned char first_error_code[2];    /* Fist error code              */
      unsigned char last_error_code[2];     /* Last error code              */
    } operational_state_chg;

    struct {
      char     operation_code;            /* Operation code                  */
                                          /* X'01' Volume movement           */
                                          /* X'02' Category selection        */
                                          /* X'03' Automatic Cartridge Loader*/
                                          /* X'04' Other                     */
      char     exception_code;            /* Exception code                  */
      char     resvd1;                    /* Reserved                        */
      unsigned char category[2];          /* VOLSER category                 */
      char     resvd2[6];                 /* Reserved                        */
      char     volser[6];                 /* VOLSER reported                 */
      char     era_code;                  /* Error recovery action code      */
      char     modifier;                  /* Modifier byte                   */
      unsigned char first_error_code[2];    /* Fist error code              */
      unsigned char last_error_code[2];     /* Last error code              */
    } volume_exception;

    struct {
      char     device_addr;               /* Device address                  */
      unsigned char atl_seqno[3];         /* Sequence number for libary      */
      char     subsys_id;                 /* Subsystem ID assigned to device */
      char     availability;              /* A (available) or U (unavailable)*/
      char     era_code;                  /* X'74 error recovery action code */
      char     modifier;                  /* X'21' - Device made unavailable */
                                          /* X'22' - Device made available   */
      unsigned char error_code[2];        /* Library error code              */
      unsigned char other_code[2];        /* Currently set to zero           */
    } device_avail_chg;

    struct {
      char     device_addr;               /* Device address                  */
      char     resvd;                     /* Reserved                        */
      unsigned char category[2];          /* Category assigned to device     */
      char     parameters;                /* Parameters set on device        */
                                          /* see fill_parm defines above     */
                                          /* these bits, same as LSDC        */
    } device_cat_chg;

    struct {
      char     audit_type;                /* Audit type                      */
      char     comp_code;                 /* Completion code                 */
      char     operation_flags;           /* Operation flags :               */
                                          /*  Bit  Description               */
                                          /*   0   Manual mode               */
                                          /*   1   ACL mount or demount      */
                                          /*   2   Logical volume audited    */
                                          /*  3-7  Reserved (set to 0)       */
      unsigned char category[2];          /* VOLSER category                 */
      char     op_volser[6];              /* Operation VOLSER                */
      char     read_volser[6];            /* VOLSER read by vision system    */
    } typeA_del_resp_msg;
  } type_spec_msg_info;                   /* Type specific information       */
};

/*****************************************************************************/
/*  Return information structure for Library Event Wait ioctl (MTIOCLEW).    */
/*****************************************************************************/
struct mtlewret {
  int      up_comp;           /* Reserved                                    */
  int      cc;                /* Completion code                             */
  int      lib_event;         /* Event detected                              */
  int      msg_type;          /* Type of message:                            */
                              /* NO_MSG         - No message                 */
                              /* UNSOL_ATTN_MSG - Unsolicited attn message   */
                              /* DELAY_RESP_MSG - Delayed response message   */
  struct   msg_info msg_info; /* Returned response message information       */
};

/*****************************************************************************/
/*  Argument structure for Library Event Wait ioctl (MTIOCLEW).              */
/*****************************************************************************/
struct  mtlewarg {
  int      flags;             /* Flags for special options                   */
  int      versn;             /* Version number                              */
  int      subcmd;            /* Subcommand :                                */
                              /*   LEWTIME - Return only when an error       */
                              /*             or exception condition is       */
                              /*             detected, or after a timeout    */
                              /*             period, whichever comes first   */
  int      timeout;           /* Timeout in seconds.  If 0 defaults to       */
                              /* 10 minutes.                                 */
  struct   mtlewret mtlewret; /* Return information                          */
};

/*****************************************************************************/
/*  Argument structure for MTIOCLDEVINFO and MTIOCLDEVLIST ioctls.           */
/*  This structure conatins an array of a maximum of 1280 devices that are   */
/*  in a given library.                                                      */
/*****************************************************************************/
#define MAXDEVICES   1280
struct mtdevinfo {
  struct {
    int      device;            /* device number                             */
    char     name[32];          /* device name                               */
  } dev[MAXDEVICES];
};

struct mtdevlist {
  struct {
    char           type[6];
    char           model[3];
    char           serial_num[8];
    unsigned char  cuid;
    unsigned char  dev;
    int            dev_number;
    int            vts_library;
  } device[MAXDEVICES];
};

/*****************************************************************************/
/*  Argument structure for MTIOCLADDR ioctl.                                 */
/*****************************************************************************/
#define MT_LIBADDR_INVALID   0      /* Address not configured                */
#define MT_LIBADDR_OFFLINE   1      /* Library is offline with this address  */
#define MT_LIBADDR_ONLINE    2      /* Library is online with this address   */

struct mtlibaddr {
      char library_name[32];        /* Logical name of library                */
      char host_ident[8];           /* Host identification for library        */
      char primary_addr[16];        /* Primary address of single library      */
      char primary_status;          /* Primary status as defined above        */
      char alternate_addr[16];      /* Alternate address of library for HA    */
      char alternate_status;        /* Alternate status as defined above      */
      char primary_addr2[16];       /* Second address of library for dual lan */
      char primary_status2;         /* Second status as defined above         */
      char alternate_addr2[16];     /* Second address of library for HA       */
      char alternate_status2;       /* Alternate status as defined above      */
    };

/*****************************************************************************/
/*                                                                           */
/*  COMPLETION CODES FOR TAPE LIBRARY IOCTL'S                                */
/*                                                                           */
/*****************************************************************************/
#define MTCC_COMPLETE         0x0000  /* Complete - No error                 */
#define MTCC_COMPLETE_VISION  0x0001  /* Complete - Vision not operational   */
#define MTCC_COMPLETE_NOTREAD 0x0002  /* Complete - VOLSER not readable      */
#define MTCC_COMPLETE_CAT     0x0003  /* Complete - Cat. assign. not changed */
#define MTCC_CANCEL_PROGREQ   0x0004  /* Cancelled - Program request         */
#define MTCC_CANCEL_ORDERSEQ  0x0005  /* Cancelled - Order sequence          */
#define MTCC_CANCEL_MANMODE   0x0006  /* Cancelled - Manual mode             */
#define MTCC_FAILED_HARDWARE  0x0007  /* Failed - Hardware                   */
#define MTCC_FAILED_VISION    0x0008  /* Failed - Vision not operational     */
#define MTCC_FAILED_NOTREAD   0x0009  /* Failed - VOLSER not readable        */
#define MTCC_FAILED_INACC     0x000A  /* Failed - Volume inaccessible        */
#define MTCC_FAILED_MISPLACED 0x000B  /* Failed - Volume misplaced           */
#define MTCC_FAILED_CATEMPTY  0x000C  /* Failed - Category empty             */
#define MTCC_FAILED_MANEJECT  0x000D  /* Failed - Volume manually ejected    */
#define MTCC_FAILED_INVENTORY 0x000E  /* Failed - Vol no longer in inventory */
#define MTCC_FAILED_NOTAVAIL  0x000F  /* Failed - Device no longer available */
#define MTCC_FAILED_LOADFAIL  0x0010  /* Failed - Permanent Load failure     */
#define MTCC_FAILED_DAMAGED   0x0011  /* Failed - Load Failure-Cart damaged  */
#define MTCC_COMPLETE_DEMOUNT 0x0012  /* Complete - Demount                  */
#define MTCC_NO_LMCP          0x0020  /* LMCP is not configured              */
#define MTCC_NOT_CMDPORT_LMCP 0x0021  /* Device is not command-port LMCP     */
#define MTCC_NO_DEV           0x0022  /* Device is not configured            */
#define MTCC_NO_DEVLIB        0x0023  /* Device is not in library            */
#define MTCC_NO_MEM           0x0024  /* Not enough memory                   */
#define MTCC_DEV_INUSE        0x0025  /* Device is in use                    */
#define MTCC_IO_FAILED        0x0026  /* I/O failed                          */
#define MTCC_DEV_INVALID      0x0027  /* Device is invalid                   */
#define MTCC_NOT_NTFPORT_LMCP 0x0028  /* Device not notification-port LMCP   */
#define MTCC_INVALID_SUBCMD   0x0029  /* Invalid subcmd parameter            */
#define MTCC_LIB_NOT_CONFIG   0x002A  /* No library device is configured     */
#define MTCC_INTERNAL_ERROR   0x002B  /* Internal error                      */
#define MTCC_INVALID_CANCELTYPE 0x002C /* Invalid cancel type                */
#define MTCC_NOT_LMCP         0x002D  /* Not an LMCP device                  */
#define MTCC_LIB_OFFLINE      0x002E  /* Library offline to host             */
#define MTCC_DRIVE_UNLOAD     0x002F  /* Volume not unloaded from drive      */
#define MTCC_COMMAND_TIMEOUT  0x0030  /* Device driver command timeout       */
#define MTCC_UNDEFINED    0xFFFFFFFF  /* Undefined completion code           */

/*****************************************************************************/
/*                                                                           */
/*  MACROS FOR LIBRARY EVENTS USED IN NOTIFICATION TYPES                     */
/*                                                                           */
/*****************************************************************************/
/*                   ERA Codes                                               */
#define MT_NTF_ERA60            0
#define MT_NTF_ERA61            1
#define MT_NTF_ERA62            2
#define MT_NTF_ERA63            3
#define MT_NTF_ERA64            4
#define MT_NTF_ERA65            5
#define MT_NTF_ERA66            6
#define MT_NTF_ERA67            7
#define MT_NTF_ERA68            8
#define MT_NTF_ERA69            9
#define MT_NTF_ERA6A            10
#define MT_NTF_ERA6B            11
#define MT_NTF_ERA6C            12
#define MT_NTF_ERA6D            13
#define MT_NTF_ERA6E            14
#define MT_NTF_ERA6F            15
#define MT_NTF_ERA70            16
#define MT_NTF_ERA71            17
#define MT_NTF_ERA72            18
#define MT_NTF_ERA73            19
#define MT_NTF_ERA74            20
#define MT_NTF_ERA75            21
#define MT_NTF_ERA76            22
#define MT_NTF_ERA77            23
#define MT_NTF_ERA78            24
#define MT_NTF_ERA79            25
#define MT_NTF_ERA7A            26
#define MT_NTF_ERA7B            27
#define MT_NTF_ERA7C            28
#define MT_NTF_ERA7D            29
#define MT_NTF_ERA7E            30
#define MT_NTF_ERA7F            31
#define MT_NTF_ERA80            32
#define MT_NTF_ERA81            33
#define MT_NTF_ERA82            34
#define MT_NTF_ERA83            35
#define MT_NTF_ERA84            36
#define MT_NTF_ERA85            37
#define MT_NTF_ERA86            38
#define MT_NTF_ERA87            39
#define MT_NTF_ERA88            40

#define MT_NTF_UNDEFERA         41

/*****************************************************************************/
/*  Unsolicited Attention Messages                                           */
/*****************************************************************************/
#define MT_NTF_ATTN_CSC         50      /* Category state change             */
#define MT_NTF_ATTN_LMOM        51      /* Library manager operator message  */
#define MT_NTF_ATTN_IOSSC       52      /* IO station state change           */
#define MT_NTF_ATTN_OSC         53      /* Operational state change          */
#define MT_NTF_ATTN_VE          54      /* Volume exception                  */
#define MT_NTF_ATTN_DAC         55      /* Device availability change        */
#define MT_NTF_ATTN_DCC         56      /* Device category change            */

/*****************************************************************************/
/*  Delayed Response Messages                                                */
/*****************************************************************************/
#define MT_NTF_DEL_MC           60      /* Mount complete                    */
#define MT_NTF_DEL_DC           61      /* Demount complete                  */
#define MT_NTF_DEL_AC           62      /* Audit complete                    */
#define MT_NTF_DEL_EC           63      /* Eject complete                    */
#define MT_NTF_DEL_LOC          64      /* Libary operation complete         */

#define MT_NTF_TIMEOUT          70      /* Timeout                           */

/*****************************************************************************/
/*                                                                           */
/*  MACROS FOR TAPE LIBRARY IOCTL'S REQUESTS                                 */
/*                                                                           */
/*****************************************************************************/
/*  Macro for IOCTL command definitions                                      */
/*****************************************************************************/
/*  For use with _IOC where no copyin and copyout are desired                */
/*  Note: _IBMR2 is the AIX/RISC 6000 version                                */
/*****************************************************************************/
#define  IOC_NOIO    0x0

#ifdef _IBMR2
#define _IOC(inout,group,num,len) \
         (inout | ((len & IOCPARM_MASK) << 16) | ((group) << 8) | (num))
#else
#ifndef __sgi
#ifndef _SEQUENT_
#ifndef __linux
#ifndef __digital__
#define _IOC(inout,group,num,len) \
         (inout | (len << 16) | (group << 8) | num)
#endif
#endif
#endif
#endif
#endif

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to mount      */
/*  a volume on a selected drive in a tape library.                          */
/*****************************************************************************/
#define MTIOCLM         _IOC(IOC_NOIO,'m',0x31,sizeof(struct mtlmarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to demount    */
/*  a volume on a selected drive in a tape library.                          */
/*****************************************************************************/
#define MTIOCLDM        _IOC(IOC_NOIO,'m',0x32,sizeof(struct mtldarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to cancel     */
/*  a queued asynchronous library operation.                                 */
/*****************************************************************************/
#define MTIOCLC         _IOC(IOC_NOIO,'m',0x33,sizeof(struct mtlcarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to change     */
/*  the category assignment of a volume.                                     */
/*****************************************************************************/
#define MTIOCLSVC       _IOC(IOC_NOIO,'m',0x34,sizeof(struct mtlsvcarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to change     */
/*  the category assignment of all volumes in one category to another        */
/*  category.                                                                */
/*****************************************************************************/
#define MTIOCLA         _IOC(IOC_NOIO,'m',0x36,sizeof(struct mtlaarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to query      */
/*  the specified library information.                                       */
/*****************************************************************************/
#define MTIOCLQ         _IOC(IOC_NOIO,'m',0x37,sizeof(struct mtlqarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts               */
/*  to query the status of a given message ID of a library operation.        */
/*****************************************************************************/
#define MTIOCLQMID      _IOC(IOC_NOIO,'m',0x38, sizeof(struct mtlqmidarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to assign     */
/*  a category to the Automatic Cartridge Loader associated with the         */
/*  device specified.                                                        */
/*****************************************************************************/
#define MTIOCLSDC       _IOC(IOC_NOIO,'m',0x39,sizeof(struct mtlsdcarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attempts to            */
/*  release a previously reserved category.                                  */
/*****************************************************************************/
#define MTIOCLRC        _IOC(IOC_NOIO,'m',0x40,sizeof(struct mtlrcarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attemps to             */
/*  reserve one or more categories.                                          */
/*****************************************************************************/
#define MTIOCLRSC       _IOC(IOC_NOIO,'m',0x41,sizeof(struct mtlrscarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that attemps to set         */
/*  a category attribute.                                                    */
/*****************************************************************************/
#define MTIOCLSCA       _IOC(IOC_NOIO,'m',0x42,sizeof(struct mtlscaarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that waits for an           */
/*  asynchronous library event to occur.                                     */
/*****************************************************************************/
#define MTIOCLEW        _IOC(IOC_NOIO,'m',0x3A,sizeof(struct mtlewarg))

/*****************************************************************************/
/*  IO control command for the ioctl system call that returns the device     */
/*  number for a given device special file                                   */
/*  This command is used on the Device special file, not the Library special */
/*  special file                                                             */
/*****************************************************************************/
#ifdef _WINNT
#define MTDEVICE        0x8402c040
#else
#define MTDEVICE        _IOR('m', 0x45, int)
#endif

/*****************************************************************************/
/*  IO control command for the ioctl system call that gets the list of       */
/*  device numbers from the library.                                         */
/*****************************************************************************/
#define MTIOCLDEVINFO   _IOC(IOC_NOIO,'m',0x48,sizeof(struct mtdevinfo))
#define MTIOCLDEVLIST   _IOC(IOC_NOIO,'m',0x49,sizeof(struct mtdevlist))

/*****************************************************************************/
/*  IO control command for the ioctl system call that gets the list of       */
/*  library addresses.                                                       */
/*****************************************************************************/
#define MTIOCLADDR      _IOC(IOC_NOIO,'m',0x50,sizeof(struct mtlibaddr))

#endif
